@@ -9,8 +9,16 @@
 
 extern MySQL_Variables mysql_variables;
 
+/* The default mariadb-connecter 3.1.4 does not yet implement CLIENT_DEPRECATE_EOF
+ * flag.
+ */
+#ifndef CLIENT_DEPRECATE_EOF
+#define CLIENT_DEPRECATE_EOF     (1UL << 24)
+#endif
+
 class MySQL_ResultSet {
 	private:
+	bool deprecate_eof_active;
 	public:
 	bool transfer_started;
 	bool resultset_completed;
@@ -20,6 +28,7 @@ class MySQL_ResultSet {
 	MySQL_Protocol *myprot;
 	MYSQL *mysql;
 	MYSQL_RES *result;
+	MYSQL_STMT *stmt;
 	unsigned int num_fields;
 	unsigned long long num_rows;
 	unsigned long long resultset_size;
@@ -27,7 +36,9 @@ class MySQL_ResultSet {
 	//PtrSizeArray *PSarrayOUT;
 	MySQL_ResultSet();
 	void init(MySQL_Protocol *_myprot, MYSQL_RES *_res, MYSQL *_my, MYSQL_STMT *_stmt=NULL);
+	void init_with_stmt();
 	~MySQL_ResultSet();
+	unsigned int add_row(MYSQL_ROWS *rows);
 	unsigned int add_row(MYSQL_ROW row);
 	unsigned int add_row2(MYSQL_ROWS *row, unsigned char *offset);
 	void add_eof();
@@ -51,6 +62,7 @@ class MySQL_Prepared_Stmt_info {
 	MySQL_Prepared_Stmt_info(unsigned char *, unsigned int);
 };
 
+uint8_t mysql_decode_length(unsigned char *ptr, uint64_t *len);
 
 class MySQL_Protocol {
 	private:
@@ -76,7 +88,7 @@ class MySQL_Protocol {
 	// - a pointer to void pointer, used to return the packet if not NULL
 	// - a pointer to unsigned int, used to return the size of the packet if not NULL 
 	// for now,  they all return true
-	bool generate_pkt_OK(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, unsigned int affected_rows, uint64_t last_insert_id, uint16_t status, uint16_t warnings, char *msg);
+	bool generate_pkt_OK(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, unsigned int affected_rows, uint64_t last_insert_id, uint16_t status, uint16_t warnings, char *msg, bool eof_identifier=false);
 	bool generate_pkt_ERR(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint16_t error_code, char *sql_state, const char *sql_message, bool track=false);
 	bool generate_pkt_EOF(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint16_t warnings, uint16_t status, MySQL_ResultSet *myrs=NULL);
 //	bool generate_COM_INIT_DB(bool send, void **ptr, unsigned int *len, char *schema);
@@ -88,11 +100,11 @@ class MySQL_Protocol {
 //	bool generate_pkt_column_count(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint64_t count);
 	bool generate_pkt_column_count(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, uint64_t count, MySQL_ResultSet *myrs=NULL);
 //	bool generate_pkt_field(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len, uint8_t sequence_id, char *schema, char *table, char *org_table, char *name, char *org_name, uint16_t charset, uint32_t column_length, uint8_t type, uint16_t flags, uint8_t decimals, bool field_list, uint64_t defvalue_length, char *defvalue);
+	bool generate_pkt_field2(void **ptr, unsigned int *len, uint8_t sequence_id, MYSQL_FIELD *field, MySQL_ResultSet *myrs);
 	bool generate_pkt_field(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, char *schema, char *table, char *org_table, char *name, char *org_name, uint16_t charset, uint32_t column_length, uint8_t type, uint16_t flags, uint8_t decimals, bool field_list, uint64_t defvalue_length, char *defvalue, MySQL_ResultSet *myrs=NULL);
 	bool generate_pkt_row(bool send, void **ptr, unsigned int *len, uint8_t sequence_id, int colnums, unsigned long *fieldslen, char **fieldstxt);
-	uint8_t generate_pkt_row3(MySQL_ResultSet *myrs, unsigned int *len, uint8_t sequence_id, int colnums, unsigned long *fieldslen, char **fieldstxt);
-//	bool generate_pkt_initial_handshake(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len);
-	bool generate_pkt_initial_handshake(bool send, void **ptr, unsigned int *len, uint32_t *thread_id);
+	uint8_t generate_pkt_row3(MySQL_ResultSet *myrs, unsigned int *len, uint8_t sequence_id, int colnums, unsigned long *fieldslen, char **fieldstxt, unsigned long rl);
+	bool generate_pkt_initial_handshake(bool send, void **ptr, unsigned int *len, uint32_t *thread_id, bool deprecate_eof_active);
 //	bool generate_statistics_response(MySQL_Data_Stream *myds, bool send, void **ptr, unsigned int *len);
 	bool generate_statistics_response(bool send, void **ptr, unsigned int *len);
 
@@ -114,5 +126,8 @@ class MySQL_Protocol {
 	stmt_execute_metadata_t * get_binds_from_pkt(void *ptr, unsigned int size, MySQL_STMT_Global_info *stmt_info, stmt_execute_metadata_t **stmt_meta);
 
 	bool generate_COM_QUERY_from_COM_FIELD_LIST(PtrSize_t *pkt);
+
+	bool verify_user_attributes(int calling_line, const char *calling_func, const unsigned char *user);
+	bool user_attributes_has_spiffe(int calling_line, const char *calling_func, const unsigned char *user);
 };
 #endif /* __CLASS_MYSQL_PROTOCOL_H */
