@@ -197,7 +197,8 @@ enum session_status {
 	SETTING_VARIABLE,
 	SETTING_MULTIPLE_VARIABLES,
 	SETTING_SET_NAMES,
-	NONE
+	SHOW_WARNINGS,
+	session_status___NONE // special marker
 };
 
 #ifdef __cplusplus
@@ -383,6 +384,32 @@ enum handle_unknown_charset {
 	HANDLE_UNKNOWN_CHARSET__MAX_HANDLE_VALUE
 };
 
+/**
+ * Enum holding the different MySQL connection errors that are used to report
+ * invalid states in the backend connections.
+ */
+enum PROXYSQL_MYSQL_ERR {
+	ER_PROXYSQL_MAX_CONN_TIMEOUT                      = 9001,
+	ER_PROXYSQL_MAX_CONN_FAILURES                     = 9002,
+	ER_PROXYSQL_COMMAND_NOT_SUPPORTED                 = 9003,
+	ER_PROXYSQL_OFFLINE_SRV                           = 9004,
+	ER_PROXYSQL_LAGGING_SRV                           = 9005,
+	ER_PROXYSQL_PING_TIMEOUT                          = 9006,
+	ER_PROXYSQL_CHANGE_USER_TIMEOUT                   = 9007,
+	ER_PROXYSQL_GR_HEALTH_CHECK_TIMEOUT               = 9008,
+	ER_PROXYSQL_GR_HEALTH_CHECKS_MISSED               = 9009,
+	ER_PROXYSQL_READ_ONLY_CHECK_CONN_TIMEOUT          = 9010,
+	ER_PROXYSQL_READ_ONLY_CHECK_TIMEOUT               = 9011,
+	ER_PROXYSQL_READ_ONLY_CHECKS_MISSED               = 9012,
+	ER_PROXYSQL_GALERA_HEALTH_CHECK_CONN_TIMEOUT      = 9013,
+	ER_PROXYSQL_GALERA_HEALTH_CHECK_TIMEOUT           = 9014,
+	ER_PROXYSQL_GALERA_HEALTH_CHECKS_MISSED           = 9015,
+	ER_PROXYSQL_AWS_NO_PINGABLE_SRV                   = 9016,
+	ER_PROXYSQL_AWS_HEALTH_CHECK_CONN_TIMEOUT         = 9017,
+	ER_PROXYSQL_AWS_HEALTH_CHECK_TIMEOUT              = 9018,
+	ER_PROXYSQL_SRV_NULL_REPLICATION_LAG              = 9019,
+};
+
 #endif /* PROXYSQL_ENUMS */
 
 
@@ -676,7 +703,7 @@ EXTERN global_variables glovars;
 #ifdef PROXYSQL_EXTERN
 #ifndef GLOBAL_DEFINED_OPTS_ENTRIES
 #define GLOBAL_DEFINED_OPTS_ENTRIES
-ProxySQL_GlobalVariables GloVars;
+ProxySQL_GlobalVariables GloVars {};
 #endif // GLOBAL_DEFINED_OPTS_ENTRIES 
 #ifndef GLOBAL_DEFINED_HOSTGROUP
 #define GLOBAL_DEFINED_HOSTGROUP
@@ -740,7 +767,6 @@ __thread bool mysql_thread___have_ssl;
 __thread bool mysql_thread___client_found_rows;
 __thread bool mysql_thread___multiplexing;
 __thread bool mysql_thread___log_unhealthy_connections;
-__thread bool mysql_thread___forward_autocommit;
 __thread bool mysql_thread___enforce_autocommit_on_reads;
 __thread bool mysql_thread___autocommit_false_not_reusable;
 __thread bool mysql_thread___autocommit_false_is_transaction;
@@ -764,6 +790,10 @@ __thread bool mysql_thread___sessions_sort;
 __thread bool mysql_thread___kill_backend_connection_when_disconnect;
 __thread bool mysql_thread___client_session_track_gtid;
 __thread char * mysql_thread___default_variables[SQL_NAME_LAST];
+__thread int mysql_thread___query_digests_grouping_limit;
+__thread bool mysql_thread___enable_client_deprecate_eof;
+__thread bool mysql_thread___enable_server_deprecate_eof;
+__thread bool mysql_thread___log_mysql_warnings_enabled;
 
 /* variables used for Query Cache */
 __thread int mysql_thread___query_cache_size_MB;
@@ -888,7 +918,6 @@ extern __thread bool mysql_thread___have_ssl;
 extern __thread bool mysql_thread___client_found_rows;
 extern __thread bool mysql_thread___multiplexing;
 extern __thread bool mysql_thread___log_unhealthy_connections;
-extern __thread bool mysql_thread___forward_autocommit;
 extern __thread bool mysql_thread___enforce_autocommit_on_reads;
 extern __thread bool mysql_thread___autocommit_false_not_reusable;
 extern __thread bool mysql_thread___autocommit_false_is_transaction;
@@ -912,6 +941,10 @@ extern __thread bool mysql_thread___sessions_sort;
 extern __thread bool mysql_thread___kill_backend_connection_when_disconnect;
 extern __thread bool mysql_thread___client_session_track_gtid;
 extern __thread char * mysql_thread___default_variables[SQL_NAME_LAST];
+extern __thread int mysql_thread___query_digests_grouping_limit;
+extern __thread bool mysql_thread___enable_client_deprecate_eof;
+extern __thread bool mysql_thread___enable_server_deprecate_eof;
+extern __thread bool mysql_thread___log_mysql_warnings_enabled;
 
 /* variables used for Query Cache */
 extern __thread int mysql_thread___query_cache_size_MB;
@@ -1003,7 +1036,7 @@ typedef struct {
 */
 mysql_variable_st mysql_tracked_variables[] {
     { SQL_CHARACTER_SET,         SETTING_CHARSET,    false, true, false,  false, false, (char *)"charset", (char *)"charset", (char *)"utf8" , true} , // should be before SQL_CHARACTER_SET_RESULTS
-    { SQL_CHARACTER_ACTION,      NONE,		         false, false, false, false, false, (char *)"action", (char *)"action", (char *)"1" , false} ,
+    { SQL_CHARACTER_ACTION,    session_status___NONE,false, false, false, false, false, (char *)"action", (char *)"action", (char *)"1" , false} ,
     { SQL_SET_NAMES,             SETTING_SET_NAMES,  false, false, false, false, false, (char *)"names", (char *)"names", (char *)"DEFAULT" , false} ,
 	{ SQL_SAFE_UPDATES,          SETTING_VARIABLE,   true,  false, true,  false, true,  (char *)"sql_safe_updates",  (char *)"sql_safe_updates", (char *)"OFF" , false} ,
     { SQL_SELECT_LIMIT,          SETTING_VARIABLE,   false, false, true,  true, false, (char *)"sql_select_limit", (char *)"sql_select_limit", (char *)"DEFAULT" , false} ,
