@@ -101,6 +101,7 @@ struct _Query_Processor_rule_t {
 	int gtid_from_hostgroup;
 	int log;
 	bool apply;
+	char* attributes;
   char *comment; // #643
 	void *regex_engine1;
 	void *regex_engine2;
@@ -133,6 +134,7 @@ class Query_Processor_Output {
 	long long max_lag_ms;
 	int log;
 	int firewall_whitelist_mode;
+	char *attributes;
 	char *comment; // #643
 	char *min_gtid;
 	bool create_new_conn;
@@ -171,6 +173,7 @@ class Query_Processor_Output {
 		new_query=NULL;
 		error_msg=NULL;
 		OK_msg=NULL;
+		attributes=NULL;
 		comment=NULL; // #643
 		min_gtid=NULL;
 		firewall_whitelist_mode = WUS_NOT_FOUND;
@@ -189,6 +192,9 @@ class Query_Processor_Output {
 			free(min_gtid);
 			min_gtid = NULL;
 		}
+		if (attributes) {
+			free(attributes);
+		}
 		if (comment) { // #643
 			free(comment);
 		}
@@ -286,7 +292,7 @@ class Query_Processor {
 	void wrlock();		// explicit write lock, to be used in multi-insert
 	void wrunlock();	// explicit write unlock
 	bool insert(QP_rule_t *qr, bool lock=true);		// insert a new rule. Uses a generic void pointer to a structure that may vary depending from the Query Processor
-	QP_rule_t * new_query_rule(int rule_id, bool active, char *username, char *schemaname, int flagIN, char *client_addr, char *proxy_addr, int proxy_port, char *digest, char *match_digest, char *match_pattern, bool negate_match_pattern, char *re_modifiers, int flagOUT, char *replace_pattern, int destination_hostgroup, int cache_ttl, int cache_empty_result, int cache_timeout, int reconnect, int timeout, int retries, int delay, int next_query_flagIN, int mirror_hostgroup, int mirror_flagOUT, char *error_msg, char *OK_msg, int sticky_conn, int multiplex, int gtid_from_hostgroup, int log, bool apply, char *comment);	// to use a generic query rule struct, this is generated by this function and returned as generic void pointer
+	QP_rule_t * new_query_rule(int rule_id, bool active, char *username, char *schemaname, int flagIN, char *client_addr, char *proxy_addr, int proxy_port, char *digest, char *match_digest, char *match_pattern, bool negate_match_pattern, char *re_modifiers, int flagOUT, char *replace_pattern, int destination_hostgroup, int cache_ttl, int cache_empty_result, int cache_timeout, int reconnect, int timeout, int retries, int delay, int next_query_flagIN, int mirror_hostgroup, int mirror_flagOUT, char *error_msg, char *OK_msg, int sticky_conn, int multiplex, int gtid_from_hostgroup, int log, bool apply, char* attributes, char *comment);	// to use a generic query rule struct, this is generated by this function and returned as generic void pointer
 	void delete_query_rule(QP_rule_t *qr);	// destructor
 	Query_Processor_Output * process_mysql_query(MySQL_Session *sess, void *ptr, unsigned int size, Query_Info *qi);
 	void delete_QP_out(Query_Processor_Output *o);
