@@ -79,7 +79,7 @@ __loop_remove_inactives:
 #endif
 }
 
-bool MySQL_Authentication::add(char * username, char * password, enum cred_username_type usertype, bool use_ssl, int default_hostgroup, char *default_schema, bool schema_locked, bool transaction_persistent, bool fast_forward, int max_connections, char *comment) {
+bool MySQL_Authentication::add(char * username, char * password, enum cred_username_type usertype, bool use_ssl, int default_hostgroup, char *default_schema, bool schema_locked, bool transaction_persistent, bool fast_forward, int max_connections, char* attributes, char *comment) {
 	uint64_t hash1, hash2;
 	SpookyHash myhash;
 	myhash.Init(1,2);
@@ -116,12 +116,41 @@ bool MySQL_Authentication::add(char * username, char * password, enum cred_usern
 			free(ad->comment);
 			ad->comment=strdup(comment);
 		}
+		if (strcasecmp(ad->attributes, attributes)) {
+			free(ad->attributes);
+			if (strlen(attributes)) {
+				// NOTE: add() is only place where we do input validation
+				try {
+					nlohmann::json valid=nlohmann::json::parse(attributes);
+					ad->attributes=strdup(attributes);
+				}
+				catch(nlohmann::json::exception& e) {
+					ad->attributes=strdup("");
+					proxy_error("Invalid attributes for user %s: %s\n", username, attributes);
+				}
+			} else {
+				ad->attributes=strdup(attributes); // default, empty string
+			}
+		}
   } else {
 		ad=(account_details_t *)malloc(sizeof(account_details_t));
 		ad->username=strdup(username);
 		ad->default_schema=strdup(default_schema);
 		ad->comment=strdup(comment);
 		ad->password=strdup(password);
+		if (strlen(attributes)) {
+			// NOTE: add() is only place where we do input validation
+			try {
+				nlohmann::json valid=nlohmann::json::parse(attributes);
+				ad->attributes=strdup(attributes);
+			}
+			catch(nlohmann::json::exception& e) {
+				ad->attributes=strdup("");
+				proxy_error("Invalid attributes for user %s: %s\n", username, attributes);
+			}
+		} else {
+			ad->attributes=strdup(attributes); // default, empty string
+		}
 		new_ad=true;
 		ad->sha1_pass=NULL;
 		ad->num_connections_used=0;
@@ -165,6 +194,7 @@ unsigned int MySQL_Authentication::memory_usage() {
 		if (ado->sha1_pass) ret += SHA_DIGEST_LENGTH;
 		if (ado->default_schema) ret += strlen(ado->default_schema) + 1;
 		if (ado->comment) ret += strlen(ado->comment) + 1;
+		if (ado->attributes) ret += strlen(ado->attributes) + 1;
 	}
 	ret += sizeof(creds_group_t);
 	ret += sizeof(PtrArray);
@@ -177,6 +207,7 @@ unsigned int MySQL_Authentication::memory_usage() {
 		if (ado->sha1_pass) ret += SHA_DIGEST_LENGTH;
 		if (ado->default_schema) ret += strlen(ado->default_schema) + 1;
 		if (ado->comment) ret += strlen(ado->comment) + 1;
+		if (ado->attributes) ret += strlen(ado->attributes) + 1;
 	}
 	ret += sizeof(creds_group_t);
 	ret += sizeof(PtrArray);
@@ -218,6 +249,7 @@ int MySQL_Authentication::dump_all_users(account_details_t ***ads, bool _complet
 		if (_complete==false) {
 			ad->password=NULL;
 			ad->default_schema=NULL;
+			ad->attributes=NULL;
 			ad->comment=NULL;
 			ad->num_connections_used=ado->num_connections_used;
 		} else {
@@ -227,6 +259,7 @@ int MySQL_Authentication::dump_all_users(account_details_t ***ads, bool _complet
 			ad->sha1_pass=NULL;
 			ad->use_ssl=ado->use_ssl;
 			ad->default_schema=strdup(ado->default_schema);
+			ad->attributes=strdup(ado->attributes);
 			ad->comment=strdup(ado->comment);
 			ad->schema_locked=ado->schema_locked;
 			ad->transaction_persistent=ado->transaction_persistent;
@@ -248,6 +281,7 @@ int MySQL_Authentication::dump_all_users(account_details_t ***ads, bool _complet
 		ad->use_ssl=ado->use_ssl;
 		ad->default_hostgroup=ado->default_hostgroup;
 		ad->default_schema=strdup(ado->default_schema);
+		ad->attributes=strdup(ado->attributes);
 		ad->comment=strdup(ado->comment);
 		ad->schema_locked=ado->schema_locked;
 		ad->transaction_persistent=ado->transaction_persistent;
@@ -361,6 +395,7 @@ bool MySQL_Authentication::del(char * username, enum cred_username_type usertype
 		free(ad->password);
 		if (ad->sha1_pass) { free(ad->sha1_pass); ad->sha1_pass=NULL; }
 		free(ad->default_schema);
+		free(ad->attributes);
 		free(ad->comment);
 		free(ad);
 		ret=true;
@@ -428,7 +463,7 @@ bool MySQL_Authentication::exists(char * username) {
 	return ret;
 }
 
-char * MySQL_Authentication::lookup(char * username, enum cred_username_type usertype, bool *use_ssl, int *default_hostgroup, char **default_schema, bool *schema_locked, bool *transaction_persistent, bool *fast_forward, int *max_connections, void **sha1_pass) {
+char * MySQL_Authentication::lookup(char * username, enum cred_username_type usertype, bool *use_ssl, int *default_hostgroup, char **default_schema, bool *schema_locked, bool *transaction_persistent, bool *fast_forward, int *max_connections, void **sha1_pass, char **attributes) {
 	char *ret=NULL;
 	uint64_t hash1, hash2;
 	SpookyHash myhash;
@@ -461,6 +496,7 @@ char * MySQL_Authentication::lookup(char * username, enum cred_username_type use
 				memcpy(*sha1_pass,ad->sha1_pass,SHA_DIGEST_LENGTH);
 			}
 		}
+		if (attributes) *attributes=l_strdup(ad->attributes);
 	}
 #ifdef PROXYSQL_AUTH_PTHREAD_MUTEX
 	pthread_rwlock_unlock(&cg.lock);
@@ -491,6 +527,7 @@ bool MySQL_Authentication::_reset(enum cred_username_type usertype) {
 			if (ad->sha1_pass) { free(ad->sha1_pass); ad->sha1_pass=NULL; }
 			free(ad->default_schema);
 			free(ad->comment);
+			free(ad->attributes);
 			free(ad);
 		}
 	}
@@ -537,6 +574,9 @@ uint64_t MySQL_Authentication::_get_runtime_checksum(enum cred_username_type use
 				myhash.Update(ad->default_schema,strlen(ad->default_schema));
 			if (ad->comment)
 				myhash.Update(ad->comment,strlen(ad->comment));
+			if (ad->attributes) {
+				myhash.Update(ad->attributes,strlen(ad->attributes));
+			}
 		}
 		it++;
 	}
