@@ -1,3 +1,4 @@
+#include "MySQL_HostGroups_Manager.h"
 #include "proxysql.h"
 #include "cpp.h"
 #include "SpookyV2.h"
@@ -11,6 +12,99 @@
 
 #include <atomic>
 
+// some of the code that follows is from mariadb client library memory allocator
+typedef int     myf;    // Type of MyFlags in my_funcs
+#define MYF(v)      (myf) (v)
+#define MY_KEEP_PREALLOC    1
+#define MY_ALIGN(A,L)    (((A) + (L) - 1) & ~((L) - 1))
+#define ALIGN_SIZE(A)    MY_ALIGN((A),sizeof(double))
+void ma_free_root(MA_MEM_ROOT *root, myf MyFLAGS);
+void *ma_alloc_root(MA_MEM_ROOT *mem_root, size_t Size);
+#define MAX(a,b) (((a) > (b)) ? (a) : (b))
+
+void * ma_alloc_root(MA_MEM_ROOT *mem_root, size_t Size)
+{
+  size_t get_size;
+  void * point;
+  MA_USED_MEM *next= 0;
+  MA_USED_MEM **prev;
+
+  Size= ALIGN_SIZE(Size);
+
+  if ((*(prev= &mem_root->free)))
+  {
+    if ((*prev)->left < Size &&
+        mem_root->first_block_usage++ >= 16 &&
+        (*prev)->left < 4096)
+    {
+      next= *prev;
+      *prev= next->next;
+      next->next= mem_root->used;
+      mem_root->used= next;
+      mem_root->first_block_usage= 0;
+    }
+    for (next= *prev; next && next->left < Size; next= next->next)
+      prev= &next->next;
+  }
+  if (! next)
+  {                     /* Time to alloc new block */
+    get_size= MAX(Size+ALIGN_SIZE(sizeof(MA_USED_MEM)),
+              (mem_root->block_size & ~1) * ( (mem_root->block_num >> 2) < 4 ? 4 : (mem_root->block_num >> 2) ) );
+
+    if (!(next = (MA_USED_MEM*) malloc(get_size)))
+    {
+      if (mem_root->error_handler)
+    (*mem_root->error_handler)();
+      return((void *) 0);               /* purecov: inspected */
+    }
+    mem_root->block_num++;
+    next->next= *prev;
+    next->size= get_size;
+    next->left= get_size-ALIGN_SIZE(sizeof(MA_USED_MEM));
+    *prev=next;
+  }
+  point= (void *) ((char*) next+ (next->size-next->left));
+  if ((next->left-= Size) < mem_root->min_malloc)
+  {                     /* Full block */
+    *prev=next->next;               /* Remove block from list */
+    next->next=mem_root->used;
+    mem_root->used=next;
+    mem_root->first_block_usage= 0;
+  }
+  return(point);
+}
+
+
+void ma_free_root(MA_MEM_ROOT *root, myf MyFlags)
+{ 
+  MA_USED_MEM *next,*old;
+
+  if (!root)
+    return; /* purecov: inspected */
+  if (!(MyFlags & MY_KEEP_PREALLOC))
+    root->pre_alloc=0;
+
+  for ( next=root->used; next ;)
+  {
+    old=next; next= next->next ;
+    if (old != root->pre_alloc)
+      free(old);
+  }
+  for (next= root->free ; next ; )
+  {
+    old=next; next= next->next ;
+    if (old != root->pre_alloc)
+      free(old);
+  }
+  root->used=root->free=0;
+  if (root->pre_alloc)
+  {
+    root->free=root->pre_alloc;
+    root->free->left=root->pre_alloc->size-ALIGN_SIZE(sizeof(MA_USED_MEM));
+    root->free->next=0;
+  }
+}
+
 extern char * binary_sha1;
 
 extern const MARIADB_CHARSET_INFO * proxysql_find_charset_nr(unsigned int nr);
@@ -168,6 +262,7 @@ void MySQL_Connection::compute_unknown_transaction_status() {
 			unknown_transaction_status = false; // no error
 			return;
 		}
+		MyHGM->p_update_mysql_error_counter(p_mysql_error_type::mysql, parent->myhgc->hid, parent->address, parent->port, _myerrno);
 		if (_myerrno >= 2000 && _myerrno < 3000) { // client error
 			// do not change it
 			return;
@@ -295,9 +390,6 @@ MySQL_Connection::MySQL_Connection() {
 	myds=NULL;
 	inserted_into_pool=0;
 	reusable=false;
-	has_prepared_statement=false;
-	processing_prepared_statement_prepare=false;
-	processing_prepared_statement_execute=false;
 	parent=NULL;
 	userinfo=new MySQL_Connection_userinfo();
 	fd=-1;
@@ -459,97 +551,18 @@ bool MySQL_Connection::is_expired(unsigned long long timeout) {
 	return false;
 }
 
-void MySQL_Connection::set_status_transaction(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_TRANSACTION;
+void MySQL_Connection::set_status(bool set, uint32_t status_flag) {
+	if (set) {
+		this->status_flags |= status_flag;
 	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_TRANSACTION;
+		this->status_flags &= ~status_flag;
 	}
 }
 
-void MySQL_Connection::set_status_compression(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_COMPRESSION;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_COMPRESSION;
-	}
+bool MySQL_Connection::get_status(uint32_t status_flag) {
+	return this->status_flags & status_flag;
 }
 
-void MySQL_Connection::set_status_get_lock(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_GET_LOCK;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_GET_LOCK;
-	}
-}
-
-void MySQL_Connection::set_status_found_rows(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_FOUND_ROWS;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_FOUND_ROWS;
-	}
-}
-
-void MySQL_Connection::set_status_lock_tables(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_LOCK_TABLES;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_LOCK_TABLES;
-	}
-}
-
-void MySQL_Connection::set_status_temporary_table(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE;
-	}
-}
-
-void MySQL_Connection::set_status_no_backslash_escapes(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_NO_BACKSLASH_ESCAPES;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_NO_BACKSLASH_ESCAPES;
-	}
-}
-
-void MySQL_Connection::set_status_user_variable(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_USER_VARIABLE;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_USER_VARIABLE;
-	}
-}
-
-void MySQL_Connection::set_status_has_savepoint(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT;
-	}
-}
-
-void MySQL_Connection::set_status_prepared_statement(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT;
-	}
-}
-
-void MySQL_Connection::set_status_no_multiplex(bool v) {
-	if (v) {
-		status_flags |= STATUS_MYSQL_CONNECTION_NO_MULTIPLEX;
-	} else {
-		status_flags &= ~STATUS_MYSQL_CONNECTION_NO_MULTIPLEX;
-	}
-}
-
-// pay attention here. set_status_sql_log_bin0 sets it sql_log_bin is ZERO
-// sql_log_bin=0 => true
-// sql_log_bin=1 => false
 void MySQL_Connection::set_status_sql_log_bin0(bool v) {
 	if (v) {
 		status_flags |= STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0;
@@ -558,50 +571,6 @@ void MySQL_Connection::set_status_sql_log_bin0(bool v) {
 	}
 }
 
-bool MySQL_Connection::get_status_transaction() {
-	return status_flags & STATUS_MYSQL_CONNECTION_TRANSACTION;
-}
-
-bool MySQL_Connection::get_status_compression() {
-	return status_flags & STATUS_MYSQL_CONNECTION_COMPRESSION;
-}
-
-bool MySQL_Connection::get_status_user_variable() {
-	return status_flags & STATUS_MYSQL_CONNECTION_USER_VARIABLE;
-}
-
-bool MySQL_Connection::get_status_has_savepoint() {
-	return status_flags & STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT;
-}
-
-bool MySQL_Connection::get_status_get_lock() {
-	return status_flags & STATUS_MYSQL_CONNECTION_GET_LOCK;
-}
-
-bool MySQL_Connection::get_status_found_rows() {
-	return status_flags & STATUS_MYSQL_CONNECTION_FOUND_ROWS;
-}
-
-bool MySQL_Connection::get_status_lock_tables() {
-	return status_flags & STATUS_MYSQL_CONNECTION_LOCK_TABLES;
-}
-
-bool MySQL_Connection::get_status_temporary_table() {
-	return status_flags & STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE;
-}
-
-bool MySQL_Connection::get_status_no_backslash_escapes() {
-	return status_flags & STATUS_MYSQL_CONNECTION_NO_BACKSLASH_ESCAPES;
-}
-
-bool MySQL_Connection::get_status_prepared_statement() {
-	return status_flags & STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT;
-}
-
-bool MySQL_Connection::get_status_no_multiplex() {
-	return status_flags & STATUS_MYSQL_CONNECTION_NO_MULTIPLEX;
-}
-
 bool MySQL_Connection::get_status_sql_log_bin0() {
 	return status_flags & STATUS_MYSQL_CONNECTION_SQL_LOG_BIN0;
 }
@@ -626,6 +595,7 @@ bool MySQL_Connection::requires_CHANGE_USER(const MySQL_Connection *client_conn)
 	}
 	return false;
 }
+
 unsigned int MySQL_Connection::number_of_matching_session_variables(const MySQL_Connection *client_conn, unsigned int& not_matching) {
 	unsigned int ret=0;
 	for (auto i = 0; i < SQL_NAME_LAST; i++) {
@@ -640,6 +610,7 @@ unsigned int MySQL_Connection::number_of_matching_session_variables(const MySQL_
 	return ret;
 }
 
+
 bool MySQL_Connection::match_tracked_options(const MySQL_Connection *c) {
 	uint32_t cf1 = options.client_flag; // own client flags
 	uint32_t cf2 = c->options.client_flag; // other client flags
@@ -752,6 +723,13 @@ void MySQL_Connection::connect_start() {
 		}
 	}
 
+	// set 'CLIENT_DEPRECATE_EOF' flag if explicitly stated by 'mysql-enable_server_deprecate_eof'.
+	// Capability is disabled by default in 'mariadb_client', so setting this option is not optional
+	// for having 'CLIENT_DEPRECATE_EOF' in the connection to be stablished.
+	if (mysql_thread___enable_server_deprecate_eof) {
+		mysql->options.client_flag |= CLIENT_DEPRECATE_EOF;
+	}
+
 	char *auth_password=NULL;
 	if (userinfo->password) {
 		if (userinfo->password[0]=='*') { // we don't have the real password, let's pass sha1
@@ -766,6 +744,18 @@ void MySQL_Connection::connect_start() {
 		async_exit_status=mysql_real_connect_start(&ret_mysql, mysql, "localhost", userinfo->username, auth_password, userinfo->schemaname, parent->port, parent->address, client_flags);
 	}
 	fd=mysql_get_socket(mysql);
+//	{
+//		// FIXME: THIS IS FOR TESTING PURPOSE ONLY
+//		// DO NOT ENABLE THIS CODE FOR PRODUCTION USE
+//		// we drastically reduce the receive buffer to make sure that
+//		// mysql_stmt_store_result_[start|continue] doesn't complete
+//		// in a single call
+//		int rcvbuf = 10240;
+//		if(setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &rcvbuf, sizeof(rcvbuf)) < 0) {
+//			proxy_error("Failed to call setsockopt\n");
+//			exit(EXIT_FAILURE);
+//		}
+//	}
 }
 
 void MySQL_Connection::connect_cont(short event) {
@@ -1029,11 +1019,13 @@ handler_again:
 			//}
 			break;
 		case ASYNC_CONNECT_FAILED:
+			MyHGM->p_update_mysql_error_counter(p_mysql_error_type::mysql, parent->myhgc->hid, parent->address, parent->port, mysql_errno(mysql));
 			parent->connect_error(mysql_errno(mysql));
 			break;
 		case ASYNC_CONNECT_TIMEOUT:
 			//proxy_error("Connect timeout on %s:%d : %llu - %llu = %llu\n",  parent->address, parent->port, myds->sess->thread->curtime , myds->wait_until, myds->sess->thread->curtime - myds->wait_until);
 			proxy_error("Connect timeout on %s:%d : exceeded by %lluus\n", parent->address, parent->port, myds->sess->thread->curtime - myds->wait_until);
+			MyHGM->p_update_mysql_error_counter(p_mysql_error_type::mysql, parent->myhgc->hid, parent->address, parent->port, mysql_errno(mysql));
 			parent->connect_error(mysql_errno(mysql));
 			break;
 		case ASYNC_CHANGE_USER_START:
@@ -1112,7 +1104,7 @@ handler_again:
 			__sync_fetch_and_add(&parent->queries_sent,1);
 			__sync_fetch_and_add(&parent->bytes_sent,query.length);
 			statuses.questions++;
-			myds->sess->thread->status_variables.queries_backends_bytes_sent+=query.length;
+			myds->sess->thread->status_variables.stvar[st_var_queries_backends_bytes_sent]+=query.length;
 			myds->bytes_info.bytes_sent += query.length;
 			bytes_info.bytes_sent += query.length;
 			if (myds->sess->with_gtid == true) {
@@ -1145,7 +1137,7 @@ handler_again:
 			stmt_prepare_start();
 			__sync_fetch_and_add(&parent->queries_sent,1);
 			__sync_fetch_and_add(&parent->bytes_sent,query.length);
-			myds->sess->thread->status_variables.queries_backends_bytes_sent+=query.length;
+			myds->sess->thread->status_variables.stvar[st_var_queries_backends_bytes_sent]+=query.length;
 			myds->bytes_info.bytes_sent += query.length;
 			bytes_info.bytes_sent += query.length;
 			if (async_exit_status) {
@@ -1179,7 +1171,7 @@ handler_again:
 			stmt_execute_start();
 			__sync_fetch_and_add(&parent->queries_sent,1);
 			__sync_fetch_and_add(&parent->bytes_sent,query.stmt_meta->size);
-			myds->sess->thread->status_variables.queries_backends_bytes_sent+=query.stmt_meta->size;
+			myds->sess->thread->status_variables.stvar[st_var_queries_backends_bytes_sent]+=query.stmt_meta->size;
 			myds->bytes_info.bytes_sent += query.stmt_meta->size;
 			bytes_info.bytes_sent += query.stmt_meta->size;
 			if (async_exit_status) {
@@ -1205,6 +1197,30 @@ handler_again:
 				query.stmt_result=mysql_stmt_result_metadata(query.stmt);
 				if (query.stmt_result==NULL) {
 					NEXT_IMMEDIATE(ASYNC_STMT_EXECUTE_END);
+				} else {
+					if (myds->sess->mirror==false) {
+						if (MyRS_reuse == NULL) {
+							MyRS = new MySQL_ResultSet();
+							MyRS->init(&myds->sess->client_myds->myprot, query.stmt_result, mysql, query.stmt);
+						} else {
+							MyRS = MyRS_reuse;
+							MyRS_reuse = NULL;
+							MyRS->init(&myds->sess->client_myds->myprot, query.stmt_result, mysql, query.stmt);
+						}
+					} else {
+/*
+						// we do not support mirroring with prepared statements
+						if (MyRS_reuse == NULL) {
+							MyRS = new MySQL_ResultSet();
+							MyRS->init(NULL, mysql_result, mysql);
+						} else {
+							MyRS = MyRS_reuse;
+							MyRS_reuse = NULL;
+							MyRS->init(NULL, mysql_result, mysql);
+						}
+*/
+					}
+					//async_fetch_row_start=false;
 				}
 			}
 			stmt_execute_store_result_start();
@@ -1215,8 +1231,47 @@ handler_again:
 			}
 			break;
 		case ASYNC_STMT_EXECUTE_STORE_RESULT_CONT:
+			{ // this copied mostly from ASYNC_USE_RESULT_CONT
+				if (myds->sess && myds->sess->client_myds && myds->sess->mirror==false) {
+					unsigned int buffered_data=0;
+					buffered_data = myds->sess->client_myds->PSarrayOUT->len * RESULTSET_BUFLEN;
+					buffered_data += myds->sess->client_myds->resultset->len * RESULTSET_BUFLEN;
+					if (buffered_data > (unsigned int)mysql_thread___threshold_resultset_size*8) {
+						next_event(ASYNC_STMT_EXECUTE_STORE_RESULT_CONT); // we temporarily pause . See #1232
+						break;
+					}
+				}
+			}
 			stmt_execute_store_result_cont(event);
+			//if (async_fetch_row_start==false) {
+			//	async_fetch_row_start=true;
+			//}
 			if (async_exit_status) {
+				// this copied mostly from ASYNC_USE_RESULT_CONT
+				MYSQL_ROWS *r=query.stmt->result.data;
+				long long unsigned int rows_read_inner = 0;
+
+				if (r) {
+					rows_read_inner++;
+					while(rows_read_inner < query.stmt->result.rows) {
+						// it is very important to check rows_read_inner FIRST
+						// because r->next could point to an invalid memory
+						rows_read_inner++;
+						r = r->next;
+					}
+					if (rows_read_inner > 1) {
+						process_rows_in_ASYNC_STMT_EXECUTE_STORE_RESULT_CONT(processed_bytes);
+						if (
+							(processed_bytes > (unsigned int)mysql_thread___threshold_resultset_size*8)
+								||
+							( mysql_thread___throttle_ratio_server_to_client && mysql_thread___throttle_max_bytes_per_second_to_client && (processed_bytes > (unsigned long long)mysql_thread___throttle_max_bytes_per_second_to_client/10*(unsigned long long)mysql_thread___throttle_ratio_server_to_client) )
+						) {
+							next_event(ASYNC_STMT_EXECUTE_STORE_RESULT_CONT); // we temporarily pause
+						} else {
+							NEXT_IMMEDIATE(ASYNC_STMT_EXECUTE_STORE_RESULT_CONT); // we continue looping
+						}
+					}
+				}
 				next_event(ASYNC_STMT_EXECUTE_STORE_RESULT_CONT);
 			} else {
 				NEXT_IMMEDIATE(ASYNC_STMT_EXECUTE_END);
@@ -1243,7 +1298,7 @@ handler_again:
 						}
 					}
 					__sync_fetch_and_add(&parent->bytes_recv,total_size);
-					myds->sess->thread->status_variables.queries_backends_bytes_recv+=total_size;
+					myds->sess->thread->status_variables.stvar[st_var_queries_backends_bytes_recv]+=total_size;
 					myds->bytes_info.bytes_recv += total_size;
 					bytes_info.bytes_recv += total_size;
 				}
@@ -1365,9 +1420,12 @@ handler_again:
 			} else {
 				async_fetch_row_start=false;
 				if (mysql_row) {
+					if (myds && myds->sess && myds->sess->status == SHOW_WARNINGS) {
+						proxy_warning("MySQL Warning. Level: [%s], Code: [%s], Message: [%s]\n", mysql_row[0], mysql_row[1], mysql_row[2]);
+					}
 					unsigned int br=MyRS->add_row(mysql_row);
 					__sync_fetch_and_add(&parent->bytes_recv,br);
-					myds->sess->thread->status_variables.queries_backends_bytes_recv+=br;
+					myds->sess->thread->status_variables.stvar[st_var_queries_backends_bytes_recv]+=br;
 					myds->bytes_info.bytes_recv += br;
 					bytes_info.bytes_recv += br;
 					processed_bytes+=br;	// issue #527 : this variable will store the amount of bytes processed during this event
@@ -1451,6 +1509,7 @@ handler_again:
 		case ASYNC_SET_AUTOCOMMIT_FAILED:
 			//fprintf(stderr,"%s\n",mysql_error(mysql));
 			proxy_error("Failed SET AUTOCOMMIT: %s\n",mysql_error(mysql));
+			MyHGM->p_update_mysql_error_counter(p_mysql_error_type::mysql, parent->myhgc->hid, parent->address, parent->port, mysql_errno(mysql));
 			break;
 		case ASYNC_SET_NAMES_START:
 			set_names_start();
@@ -1480,6 +1539,7 @@ handler_again:
 		case ASYNC_SET_NAMES_FAILED:
 			//fprintf(stderr,"%s\n",mysql_error(mysql));
 			proxy_error("Failed SET NAMES: %s\n",mysql_error(mysql));
+			MyHGM->p_update_mysql_error_counter(p_mysql_error_type::mysql, parent->myhgc->hid, parent->address, parent->port, mysql_errno(mysql));
 			break;
 		case ASYNC_INITDB_START:
 			initdb_start();
@@ -1508,6 +1568,7 @@ handler_again:
 			break;
 		case ASYNC_INITDB_FAILED:
 			proxy_error("Failed INITDB: %s\n",mysql_error(mysql));
+			MyHGM->p_update_mysql_error_counter(p_mysql_error_type::mysql, parent->myhgc->hid, parent->address, parent->port, mysql_errno(mysql));
 			//fprintf(stderr,"%s\n",mysql_error(mysql));
 			break;
 		case ASYNC_SET_OPTION_START:
@@ -1537,6 +1598,7 @@ handler_again:
 			break;
 		case ASYNC_SET_OPTION_FAILED:
 			proxy_error("Error setting MYSQL_OPTION_MULTI_STATEMENTS : %s\n", mysql_error(mysql));
+			MyHGM->p_update_mysql_error_counter(p_mysql_error_type::mysql, parent->myhgc->hid, parent->address, parent->port, mysql_errno(mysql));
 			break;
 
 		default:
@@ -1546,6 +1608,70 @@ handler_again:
 	return async_state_machine;
 }
 
+void MySQL_Connection::process_rows_in_ASYNC_STMT_EXECUTE_STORE_RESULT_CONT(unsigned long long& processed_bytes) {
+	// there is more than 1 row
+	unsigned long long total_size=0;
+	long long unsigned int irs = 0;
+	MYSQL_ROWS *ir = query.stmt->result.data;
+	for (irs = 0; irs < query.stmt->result.rows -1 ; irs++) {
+		// while iterating the rows we also count the bytes
+		total_size+=ir->length;
+		if (ir->length > 0xFFFFFF) {
+			total_size+=(ir->length / 0xFFFFFF) * sizeof(mysql_hdr);
+		}
+		total_size+=sizeof(mysql_hdr);
+		// add the row to the resulset
+		unsigned int br=MyRS->add_row(ir);
+		// increment counters for the bytes processed
+		__sync_fetch_and_add(&parent->bytes_recv,br);
+		myds->sess->thread->status_variables.stvar[st_var_queries_backends_bytes_recv]+=br;
+		myds->bytes_info.bytes_recv += br;
+		bytes_info.bytes_recv += br;
+		processed_bytes+=br;	// issue #527 : this variable will store the amount of bytes processed during this event
+
+		// we stop when we 'ir->next' will be pointing to the last row
+		if (irs <= query.stmt->result.rows - 2) {
+			ir = ir->next;
+		}
+	}
+	// at this point, ir points to the last row
+	// next, we create a new MYSQL_ROWS that is a copy of the last row
+	MYSQL_ROWS *lcopy = (MYSQL_ROWS *)malloc(sizeof(MYSQL_ROWS) + ir->length);
+	lcopy->length = ir->length;
+	lcopy->data= (MYSQL_ROW)(lcopy + 1);
+	memcpy((char *)lcopy->data, (char *)ir->data, ir->length);
+	// next we proceed to reset all the buffer
+
+	// this invalidates the local variables inside the coroutines
+	// pointing to the previous allocated memory for 'stmt->result'.
+	// For more context see: #3324
+	ma_free_root(&query.stmt->result.alloc, MYF(MY_KEEP_PREALLOC));
+	query.stmt->result.data= NULL;
+	query.stmt->result_cursor= NULL;
+	query.stmt->result.rows = 0;
+
+	// we will now copy back the last row and make it the only row available
+	MYSQL_ROWS *current = (MYSQL_ROWS *)ma_alloc_root(&query.stmt->result.alloc, sizeof(MYSQL_ROWS) + lcopy->length);
+	current->data= (MYSQL_ROW)(current + 1);
+	// update 'stmt->result.data' to the new allocated memory and copy the backed last row
+	query.stmt->result.data = current;
+	memcpy((char *)current->data, (char *)lcopy->data, lcopy->length);
+	// update the 'current->length' with the length of the copied row
+	current->length = lcopy->length;
+
+	// we free the copy
+	free(lcopy);
+	// change the rows count to 1
+	query.stmt->result.rows = 1;
+	// we should also configure the cursor, but because we scan it using our own
+	// algorithm, this is not needed
+
+	// now we update bytes counter
+	__sync_fetch_and_add(&parent->bytes_recv,total_size);
+	myds->sess->thread->status_variables.stvar[st_var_queries_backends_bytes_recv]+=total_size;
+	myds->bytes_info.bytes_recv += total_size;
+	bytes_info.bytes_recv += total_size;
+}
 
 void MySQL_Connection::next_event(MDB_ASYNC_ST new_st) {
 #ifdef DEBUG
@@ -2045,13 +2171,14 @@ bool MySQL_Connection::IsActiveTransaction() {
 			ret = true;
 		}
 		if (ret == false) {
-			bool r = ( mysql_thread___autocommit_false_is_transaction || mysql_thread___forward_autocommit );
+			//bool r = ( mysql_thread___autocommit_false_is_transaction || mysql_thread___forward_autocommit ); // deprecated , see #3253
+			bool r = ( mysql_thread___autocommit_false_is_transaction);
 			if ( r && (IsAutoCommit() == false) ) {
 				ret = true;
 			}
 		}
 		if (ret == false) {
-			if (get_status_has_savepoint()) {
+			if (get_status(STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT)) {
 				// there are savepoints
 				ret = true;
 			}
@@ -2187,16 +2314,16 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 			if (myds->sess->qpo) {
 				mul=myds->sess->qpo->multiplex;
 				if (mul==0) {
-					set_status_no_multiplex(true);
+					set_status(true, STATUS_MYSQL_CONNECTION_NO_MULTIPLEX);
 				} else {
 					if (mul==1) {
-						set_status_no_multiplex(false);
+						set_status(false, STATUS_MYSQL_CONNECTION_NO_MULTIPLEX);
 					}
 				}
 			}
 		}
 	}
-	if (get_status_user_variable()==false) { // we search for variables only if not already set
+	if (get_status(STATUS_MYSQL_CONNECTION_USER_VARIABLE)==false) { // we search for variables only if not already set
 //			if (
 //				strncasecmp(query_digest_text,"SELECT @@tx_isolation", strlen("SELECT @@tx_isolation"))
 //				&&
@@ -2209,12 +2336,12 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 					if (mul!=2) {
 						if (index(query_digest_text,'@')) { // mul = 2 has a special meaning : do not disable multiplex for variables in THIS QUERY ONLY
 							if (!IsKeepMultiplexEnabledVariables(query_digest_text)) {
-								set_status_user_variable(true);
+								set_status(true, STATUS_MYSQL_CONNECTION_USER_VARIABLE);
 							}
 						} else {
 							for (unsigned int i = 0; i < sizeof(session_vars)/sizeof(char *); i++) {
 								if (strcasestr(query_digest_text,session_vars[i])!=NULL)  {
-									set_status_user_variable(true);
+									set_status(true, STATUS_MYSQL_CONNECTION_USER_VARIABLE);
 									break;
 								}
 							}
@@ -2224,7 +2351,7 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 				case 1: // new algorithm
 					if (myds->sess->locked_on_hostgroup > -1) {
 						// locked_on_hostgroup was set, so some variable wasn't parsed
-						set_status_user_variable(true);
+						set_status(true, STATUS_MYSQL_CONNECTION_USER_VARIABLE);
 					}
 					break;
 				default:
@@ -2233,47 +2360,47 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 		} else {
 			if (mul!=2 && index(query_digest_text,'@')) { // mul = 2 has a special meaning : do not disable multiplex for variables in THIS QUERY ONLY
 				if (!IsKeepMultiplexEnabledVariables(query_digest_text)) {
-					set_status_user_variable(true);
+					set_status(true, STATUS_MYSQL_CONNECTION_USER_VARIABLE);
 				}
 			}
 		}
 	}
-	if (get_status_prepared_statement()==false) { // we search if prepared was already executed
+	if (get_status(STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT)==false) { // we search if prepared was already executed
 		if (!strncasecmp(query_digest_text,"PREPARE ", strlen("PREPARE "))) {
-			set_status_prepared_statement(true);
+			set_status(true, STATUS_MYSQL_CONNECTION_PREPARED_STATEMENT);
 		}
 	}
-	if (get_status_temporary_table()==false) { // we search for temporary if not already set
+	if (get_status(STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE)==false) { // we search for temporary if not already set
 		if (!strncasecmp(query_digest_text,"CREATE TEMPORARY TABLE ", strlen("CREATE TEMPORARY TABLE "))) {
-			set_status_temporary_table(true);
+			set_status(true, STATUS_MYSQL_CONNECTION_TEMPORARY_TABLE);
 		}
 	}
-	if (get_status_lock_tables()==false) { // we search for lock tables only if not already set
+	if (get_status(STATUS_MYSQL_CONNECTION_LOCK_TABLES)==false) { // we search for lock tables only if not already set
 		if (!strncasecmp(query_digest_text,"LOCK TABLE", strlen("LOCK TABLE"))) {
-			set_status_lock_tables(true);
+			set_status(true, STATUS_MYSQL_CONNECTION_LOCK_TABLES);
 		}
 	}
-	if (get_status_lock_tables()==false) { // we search for lock tables only if not already set
+	if (get_status(STATUS_MYSQL_CONNECTION_LOCK_TABLES)==false) { // we search for lock tables only if not already set
 		if (!strncasecmp(query_digest_text,"FLUSH TABLES WITH READ LOCK", strlen("FLUSH TABLES WITH READ LOCK"))) { // issue 613
-			set_status_lock_tables(true);
+			set_status(true, STATUS_MYSQL_CONNECTION_LOCK_TABLES);
 		}
 	}
-	if (get_status_lock_tables()==true) {
+	if (get_status(STATUS_MYSQL_CONNECTION_LOCK_TABLES)==true) {
 		if (!strncasecmp(query_digest_text,"UNLOCK TABLES", strlen("UNLOCK TABLES"))) {
-			set_status_lock_tables(false);
+			set_status(false, STATUS_MYSQL_CONNECTION_LOCK_TABLES);
 		}
 	}
-	if (get_status_get_lock()==false) { // we search for get_lock if not already set
+	if (get_status(STATUS_MYSQL_CONNECTION_GET_LOCK)==false) { // we search for get_lock if not already set
 		if (strcasestr(query_digest_text,"GET_LOCK(")) {
-			set_status_get_lock(true);
+			set_status(true, STATUS_MYSQL_CONNECTION_GET_LOCK);
 		}
 	}
-	if (get_status_found_rows()==false) { // we search for SQL_CALC_FOUND_ROWS if not already set
+	if (get_status(STATUS_MYSQL_CONNECTION_FOUND_ROWS)==false) { // we search for SQL_CALC_FOUND_ROWS if not already set
 		if (strcasestr(query_digest_text,"SQL_CALC_FOUND_ROWS")) {
-			set_status_found_rows(true);
+			set_status(true, STATUS_MYSQL_CONNECTION_FOUND_ROWS);
 		}
 	}
-	if (get_status_has_savepoint()==false) {
+	if (get_status(STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT)==false) {
 		if (mysql) {
 			if (
 				(mysql->server_status & SERVER_STATUS_IN_TRANS)
@@ -2281,7 +2408,7 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 				((mysql->server_status & SERVER_STATUS_AUTOCOMMIT) == 0)
 			) {
 				if (!strncasecmp(query_digest_text,"SAVEPOINT ", strlen("SAVEPOINT "))) {
-					set_status_has_savepoint(true);
+					set_status(true, STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT);
 				}
 			}
 		}
@@ -2299,7 +2426,7 @@ void MySQL_Connection::ProcessQueryAndSetStatusFlags(char *query_digest_text) {
 			||
 			!strcasecmp(query_digest_text,"ROLLBACK")
 		) {
-			set_status_has_savepoint(false);
+			set_status(false, STATUS_MYSQL_CONNECTION_HAS_SAVEPOINT);
 		}
 	}
 }
@@ -2457,7 +2584,7 @@ bool MySQL_Connection::get_gtid(char *buff, uint64_t *trx_id) {
 						// copy to external buffer in MySQL_Backend
 						memcpy(buff,data,length);
 						buff[length]=0;
-						__sync_fetch_and_add(&myds->sess->thread->status_variables.gtid_session_collected,1);
+						__sync_fetch_and_add(&myds->sess->thread->status_variables.stvar[st_var_gtid_session_collected],1);
 						ret = true;
 					}
 				}
