@@ -8,6 +8,8 @@
 #include "MySQL_PreparedStatement.h"
 #include "MySQL_Data_Stream.h"
 
+#include <openssl/x509v3.h>
+
 /*
 
 in libssl 1.1.0 
@@ -163,6 +165,31 @@ enum sslstatus MySQL_Data_Stream::do_ssl_handshake() {
 	char buf[MY_SSL_BUFFER];
 	enum sslstatus status;
 	int n = SSL_do_handshake(ssl);
+	if (n == 1) {
+		//proxy_info("SSL handshake completed\n");
+		long rc = SSL_get_verify_result(ssl);
+		if (rc != X509_V_OK && rc != X509_V_ERR_SELF_SIGNED_CERT_IN_CHAIN && rc != X509_V_ERR_UNABLE_TO_VERIFY_LEAF_SIGNATURE) {
+			proxy_error("Disconnecting %s:%d: X509 client SSL certificate verify error: (%d:%s)\n" , addr.addr, addr.port, rc, X509_verify_cert_error_string(rc));
+			return SSLSTATUS_FAIL;
+		} else {
+			X509 *cert;
+			cert = SSL_get_peer_certificate(ssl);
+			if (cert) {
+				ASN1_STRING *str;
+				GENERAL_NAME *sanName;
+				STACK_OF(GENERAL_NAME) *san_names = NULL;
+				san_names = (stack_st_GENERAL_NAME *)X509_get_ext_d2i((X509 *) cert, NID_subject_alt_name, NULL, NULL);
+				if (san_names) {
+					sanName = sk_GENERAL_NAME_value(san_names, 0);
+					str = sanName->d.dNSName;
+					proxy_info("%s\n" , str->data);
+					x509_subject_alt_name = strdup((const char*)str->data);
+				}
+			} else {
+				proxy_error("X509 error: no required certificate sent by client\n");
+			}
+		}
+	}
 	status = get_sslstatus(ssl, n);
 	//proxy_info("SSL status = %d\n", status);
 	/* Did SSL request to write bytes? */
@@ -235,6 +262,7 @@ MySQL_Data_Stream::MySQL_Data_Stream() {
 	encrypted=false;
 	switching_auth_stage = 0;
 	switching_auth_type = 0;
+	x509_subject_alt_name=NULL;
 	ssl=NULL;
 	rbio_ssl = NULL;
 	wbio_ssl = NULL;
@@ -347,6 +375,10 @@ MySQL_Data_Stream::~MySQL_Data_Stream() {
 		CompPktOUT.pkt.ptr=NULL;
 		CompPktOUT.pkt.size=0;
 	}
+	if (x509_subject_alt_name) {
+		free(x509_subject_alt_name);
+		x509_subject_alt_name=NULL;
+	}
 }
 
 // this function initializes a MySQL_Data_Stream 
@@ -674,7 +706,7 @@ int MySQL_Data_Stream::write_to_net() {
 		if (myds_type == MYDS_FRONTEND) {
 			if (sess) {
 				if (sess->thread) {
-					sess->thread->status_variables.queries_frontends_bytes_sent += bytes_io;
+					sess->thread->status_variables.stvar[st_var_queries_frontends_bytes_sent] += bytes_io;
 				}
 			}
 		}
@@ -885,7 +917,7 @@ int MySQL_Data_Stream::buffer2array() {
 		return ret;
 	}
 
-	if (myconn->get_status_compression()==true) {
+	if (myconn->get_status(STATUS_MYSQL_CONNECTION_COMPRESSION)==true) {
 		if ((queueIN.pkt.size==0) && queue_data(queueIN)>=7) {
 			proxy_debug(PROXY_DEBUG_PKT_ARRAY, 5, "Session=%p . Reading the header of a new compressed packet\n", sess);
  			memcpy(&queueIN.hdr,queue_r_ptr(queueIN), sizeof(mysql_hdr));
@@ -928,7 +960,7 @@ int MySQL_Data_Stream::buffer2array() {
 		ret+=b;
 	}
 	if ((queueIN.pkt.size>0) && (queueIN.pkt.size==queueIN.partial) ) {
-		if (myconn->get_status_compression()==true) {
+		if (myconn->get_status(STATUS_MYSQL_CONNECTION_COMPRESSION)==true) {
 			Bytef *dest = NULL;
 			uLongf destLen;
 			proxy_debug(PROXY_DEBUG_PKT_ARRAY, 5, "Session=%p . Copied the whole compressed packet\n", sess);
@@ -1153,7 +1185,7 @@ int MySQL_Data_Stream::array2buffer() {
 					queueOUT.pkt.ptr=NULL;
 				}
 		//VALGRIND_ENABLE_ERROR_REPORTING;
-				if (myconn->get_status_compression()==true) {
+				if (myconn->get_status(STATUS_MYSQL_CONNECTION_COMPRESSION)==true) {
 					proxy_debug(PROXY_DEBUG_PKT_ARRAY, 5, "Session=%p . DataStream: %p -- Compression enabled\n", sess, this);
 					generate_compressed_packet();	// it is copied directly into queueOUT.pkt					
 				} else {
@@ -1167,12 +1199,12 @@ int MySQL_Data_Stream::array2buffer() {
 						// enable compression
 						if (myconn->options.server_capabilities & CLIENT_COMPRESS) {
 							if (myconn->options.compression_min_length) {
-								myconn->set_status_compression(true);
+								myconn->set_status(true, STATUS_MYSQL_CONNECTION_COMPRESSION);
 							}
 						} else {
 							//explicitly disable compression
 							myconn->options.compression_min_length=0;
-							myconn->set_status_compression(false);
+							myconn->set_status(false, STATUS_MYSQL_CONNECTION_COMPRESSION);
 						}
 					}
 				}
