@@ -1,14 +1,17 @@
 #define PROXYSQL_EXTERN
-
+#define MAIN_PROXY_SQLITE3
 #include <stdlib.h>
 #include "tap.h"
 #include <cstdint>
 #include <cstring>
 #include <memory>
+#include <openssl/ssl.h>
+#include "proxysql_structs.h"
+#include "proxysql_glovars.hpp"
 #include "sqlite3db.h"
-#include "proxysql.h"
 
 int main() {
+	SQLite3DB::LoadPlugin(NULL);
 	plan(9);
 
 	{
