@@ -52,22 +52,15 @@ Usage example:
 
 This test script expects a certain directory layout for the workdir.
 
-It is assumed that ProxySQL has been installed.
-
   <workdir>/
-      bats/ (BATS installation)
-      logs/
-      proxysql-admin (symbolic link to SRCDIR/proxysql-admin)
-      proxysql-admin-common (symbolic link to SRCDIR/proxysql-admin-common)
-      proxysql-admin-openssl (symbolic link to SRCDIR/proxysql-admin-openssl)
+      proxysql-admin
       Percona-XtraDB-Cluster-XXX.tar.gz
       proxysql-2.0/
         etc/
-          proxysql-admin.cnf (symbolic link to SRCDIR/proxysql-admin.cnf)
+          proxysql-admin.cnf
         usr/
           bin/
-            proxysql (symbolic link to /usr/bin/proxysql)
-      proxysql_db/
+            proxysql
 
 
 The log files and datadirs may also be found in the <workdir>
@@ -303,10 +296,6 @@ function start_pxc_node(){
   echo "[sst]" >> my.cnf
   echo "wsrep_debug=1" >> my.cnf
 
-  if [[ $USE_IPVERSION == "v6" ]]; then
-    echo "sockopt=\"pf=ip6\"" >> my.cnf
-  fi
-
   # Assume that node1 is the bootstrapped node
   bootstrap_node="${PXC_BASEDIR}/${cluster_name}1"
 
